# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import builtins
import typing

@typing.final
class AppendCondition:
    r"""
    Python wrapper for DCBAppendCondition
    """
    def __new__(cls, fail_if_events_match: Query, after: typing.Optional[builtins.int] = None) -> AppendCondition: ...
    def __repr__(self) -> builtins.str: ...

@typing.final
class Client:
    r"""
    Python wrapper for the synchronous UmaDB client
    """
    def __new__(cls, url: builtins.str, ca_path: typing.Optional[builtins.str] = None, batch_size: typing.Optional[builtins.int] = None, api_key: typing.Optional[builtins.str] = None) -> Client:
        r"""
        Create a new UmaDB client
        
        Args:
            url: The server URL (e.g., "http://localhost:50051" or "https://server:50051")
            ca_path: Optional path to CA certificate for TLS
            batch_size: Optional batch size for reading events
            api_key: Optional API key for authenticating clients
        
        
        Returns:
            A connected UmaDB client
        """
    def read(self, query: typing.Optional[Query] = None, start: typing.Optional[builtins.int] = None, backwards: builtins.bool = False, limit: typing.Optional[builtins.int] = None, subscribe: builtins.bool = False) -> ReadResponse:
        r"""
        Read events from the event store
        
        Args:
            query: Optional Query to filter events
            start: Optional starting position
            backwards: Whether to read backwards (default: False)
            limit: Optional maximum number of events to read
            subscribe: Whether to subscribe to new events (default: False)
        
        Returns:
            List of SequencedEvent objects
        """
    def head(self) -> typing.Optional[builtins.int]:
        r"""
        Get the current head position of the event store
        
        Returns:
            Optional position (None if store is empty)
        """
    def append(self, events: typing.Sequence[Event], condition: typing.Optional[AppendCondition] = None) -> builtins.int:
        r"""
        Append events to the event store
        
        Args:
            events: List of Event objects to append
            condition: Optional AppendCondition
        
        Returns:
            Position of the last appended event
        """
    def __repr__(self) -> builtins.str: ...

@typing.final
class Event:
    r"""
    Python wrapper for DCBEvent
    """
    @property
    def event_type(self) -> builtins.str: ...
    @property
    def data(self) -> bytes: ...
    @property
    def tags(self) -> builtins.list[builtins.str]: ...
    @property
    def uuid(self) -> typing.Optional[builtins.str]: ...
    def __new__(cls, event_type: builtins.str, data: typing.Sequence[builtins.int], tags: typing.Optional[typing.Sequence[builtins.str]] = None, uuid: typing.Optional[builtins.str] = None) -> Event: ...
    def __repr__(self) -> builtins.str: ...

@typing.final
class Query:
    r"""
    Python wrapper for DCBQuery
    """
    def __new__(cls, items: typing.Optional[typing.Sequence[QueryItem]] = None) -> Query: ...
    def __repr__(self) -> builtins.str: ...

@typing.final
class QueryItem:
    r"""
    Python wrapper for DCBQueryItem
    """
    def __new__(cls, types: typing.Optional[typing.Sequence[builtins.str]] = None, tags: typing.Optional[typing.Sequence[builtins.str]] = None) -> QueryItem: ...
    def __repr__(self) -> builtins.str: ...

@typing.final
class ReadResponse:
    r"""
    Python iterator over sequenced events
    """
    def __iter__(self) -> ReadResponse: ...
    def __next__(self) -> SequencedEvent: ...
    def head(self) -> typing.Optional[builtins.int]:
        r"""
        Returns the current head position of the event store, or None if empty
        """
    def collect_with_head(self) -> tuple[builtins.list[SequencedEvent], typing.Optional[builtins.int]]:
        r"""
        Collects all remaining events along with the head position
        """
    def next_batch(self) -> builtins.list[SequencedEvent]:
        r"""
        Returns the next batch of events for this read. If there are no more events, returns an empty list.
        """

@typing.final
class SequencedEvent:
    r"""
    Python wrapper for DCBSequencedEvent
    """
    @property
    def event(self) -> Event: ...
    @property
    def position(self) -> builtins.int: ...
    def __repr__(self) -> builtins.str: ...



class IntegrityError(ValueError):
    """Raised when the event store detects integrity violations."""
    ...

class TransportError(RuntimeError):
    """Raised when the client fails to communicate with the server."""
    ...

class CorruptionError(RuntimeError):
    """Raised when on-disk data corruption is detected."""
    ...

class AuthenticationError(RuntimeError):
    """Raised when on-disk data corruption is detected."""
    ...



#!/usr/bin/env python3
"""
Basic usage example for UmaDB Python client.

This example demonstrates:
1. Connecting to a UmaDB server
2. Appending events
3. Reading events
4. Using queries to filter events
5. Using append conditions
"""
import os
import uuid

from umadb import Client, Event, Query, QueryItem, AppendCondition, IntegrityError


def main() -> None:
    # Connect to UmaDB server (make sure server is running)
    api_key = os.environ.get("UMADB_API_KEY")
    ca_path = os.environ.get("UMADB_TLS_CERT")
    if ca_path:
        url = "https://localhost:50051"
    else:
        url = "http://localhost:50051"
    print("Connecting to UmaDB server...")
    client = Client(url, ca_path=ca_path, api_key=api_key, batch_size=2)

    # Get current head position
    head = client.head()
    print(f"Current head position: {head}")

    # Create and append some events
    print("\nAppending events...")
    events = [
        Event(
            event_type="UserCreated",
            data=b'{"user_id": "123", "name": "Alice"}',
            tags=["user", "user:123"],
        ),
        Event(
            event_type="UserUpdated",
            data=b'{"user_id": "123", "email": "alice@example.com"}',
            tags=["user", "user:123"],
        ),
        Event(
            event_type="OrderCreated",
            data=b'{"order_id": "456", "user_id": "123"}',
            tags=["order", "user:123", "order:456"],
        ),
    ]

    position = client.append(events)
    print(f"Events appended, last position: {position}")

    # Read all events
    print("\nReading all events...")
    all_events = client.read()
    print(f"  - read response head is now: {all_events.head()}")
    for seq_event in all_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type} - tags: {seq_event.event.tags}")
        print(f"  - read response head is now: {all_events.head()}")
    print(f"  Last known position is: {all_events.head()}")

    # Read with query to filter events
    print("\nReading user-related events...")
    user_query_item = QueryItem(tags=["user"])
    user_query = Query(items=[user_query_item])
    user_events = client.read(query=user_query)
    print(f"  - read response head is now: {user_events.head()}")
    for seq_event in user_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {user_events.head()}")
    print(f"  Last known position is: {user_events.head()}")

    # Read with multiple filters (OR logic)
    print("\nReading UserCreated OR OrderCreated events...")
    query = Query(items=[
        QueryItem(types=["UserCreated"]),
        QueryItem(types=["OrderCreated"]),
    ])
    filtered_events = client.read(query=query)
    print(f"  - read response head is now: {filtered_events.head()}")
    for seq_event in filtered_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {filtered_events.head()}")
    print(f"  Last known position is: {filtered_events.head()}")

    # Read with limit forwards
    print("\nReading first 2 events forwards from the start...")
    recent_events = client.read(limit=2)
    print(f"  - read response head is now: {recent_events.head()}")
    for seq_event in recent_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {recent_events.head()}")
    print(f"  Last known position is: {recent_events.head()}")

    # Read with limit forwards
    print("\nReading first 3 events forwards from the start...")
    recent_events = client.read(limit=3)
    print(f"  - read response head is now: {recent_events.head()}")
    for seq_event in recent_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {recent_events.head()}")
    print(f"  Last known position is: {recent_events.head()}")

    # Read with limit backwards
    print("\nReading last 2 events backwards from the end...")
    old_events = client.read(limit=2, backwards=True)
    print(f"  - read response head is now: {old_events.head()}")
    for seq_event in old_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {old_events.head()}")
    print(f"  Last known position is: {old_events.head()}")

    # Read backwards without limit
    print("\nReading all events backwards from the end...")
    backwards_events = client.read(backwards=True)
    print(f"  - read response head is now: {backwards_events.head()}")
    for seq_event in backwards_events:
        print(f"  Position {seq_event.position}: {seq_event.event.event_type}")
        print(f"  - read response head is now: {backwards_events.head()}")
    print(f"  Last known position is: {backwards_events.head()}")

    # Try conditional append (preventing duplicate UserCreated for user:123)
    print("\nTrying conditional append (should fail)...")
    fail_query = Query(items=[
        QueryItem(types=["UserCreated"], tags=["user:123"])
    ])
    condition = AppendCondition(fail_if_events_match=fail_query)

    duplicate_event = Event(
        event_type="UserCreated",
        data=b'{"user_id": "123", "name": "Alice Again"}',
        tags=["user", "user:123"],
    )

    try:
        client.append([duplicate_event], condition=condition)
        print("  Unexpected: append succeeded")
    except IntegrityError as e:
        print(f"  Expected: append failed - {e}")

    # Conditional append that should succeed
    print("\nTrying conditional append for new user (should succeed)...")
    new_user_id = str(uuid.uuid4())
    new_user_event = Event(
        event_type="UserCreated",
        data=b'{"user_id": "456", "name": "Bob"}',
        tags=["user", new_user_id],
    )
    fail_query = Query(items=[
        QueryItem(types=["UserCreated"], tags=[new_user_id])
    ])
    condition = AppendCondition(fail_if_events_match=fail_query)

    try:
        position = client.append([new_user_event], condition=condition)
        print(f"  Success: event appended at position {position}")
    except ValueError as e:
        print(f"  Failed: {e}")

    # Final head position
    final_head = client.head()
    print(f"\nFinal head position: {final_head}")


if __name__ == "__main__":
    main()



"""
DON"T RUN THIS DIRECTLY: USE `make maturin-python-stubs` from the repo root.

Generate .pyi stubs for the umadb Python package, using py03-stub-gen crate.
Requires the extension to be already built and installed in the current venv.
"""
import os
from umadb._umadb import _generate_umadb_pyi_stubs  # type: ignore


STUB_PATH = "./umadb-python/python/umadb/_umadb.pyi"

os.environ["CARGO_MANIFEST_DIR"] = "./umadb-python"  # for the py03-stub-gen crate

def append_exception_stubs():
    """
    Appends stubs for the exception class. Don't know how else to do this.
    """
    exception_stubs = """
class IntegrityError(ValueError):
    \"\"\"Raised when the event store detects integrity violations.\"\"\"
    ...

class TransportError(RuntimeError):
    \"\"\"Raised when the client fails to communicate with the server.\"\"\"
    ...

class CorruptionError(RuntimeError):
    \"\"\"Raised when on-disk data corruption is detected.\"\"\"
    ...

class AuthenticationError(RuntimeError):
    \"\"\"Raised when on-disk data corruption is detected.\"\"\"
    ...
"""
    with open(STUB_PATH, "a", encoding="utf8") as f:
        f.write("\n")
        f.write(exception_stubs)

def fix_generated_stubs():
    """
    Fix the ReadResponse.__next__() method stub. Don't know how else to do this.
    """

    lines = []
    with open(STUB_PATH, "r", encoding="utf8") as f:
        lines = f.readlines()

    for i, line in enumerate(lines):
        if line == "    def __next__(self) -> typing.Optional[SequencedEvent]: ...\n":
            lines[i] = line.replace("typing.Optional[SequencedEvent]", "SequencedEvent")

    with open(STUB_PATH, "wt", encoding="utf8") as f:
        f.writelines(lines)

def main():
    _generate_umadb_pyi_stubs()
    append_exception_stubs()
    fix_generated_stubs()

if __name__ == "__main__":
    main()

